#include "hal_data.h"
#include "coaster.h"


// period16th.   This is the period in ms of a 16th note.  Equals (60sec/(bpm*16))*1000
// where bpm is the beats per minute of a given song
// 31ms corresponds to 120bpm
#define PERIOD16TH 28
//#define TEMPO 116
#define SongLength 22


ioport_level_t level2 = IOPORT_LEVEL_LOW;



//static uint8_t amp_enable = 1;
ioport_port_pin_t Speaker_pin = IOPORT_PORT_00_PIN_14;

uint8_t note_index = 0;
timer_size_t note[SongLength];
uint32_t note_duration[SongLength];

// O' Canada short version
// set #define SongLength 55
                char notes[SongLength] =
                {"faadefgabef#gabccbbaefgfefgagfgabagfeefgfefgagffeaa#fga"};
                        // 1         2         3         4         5         6         7         8         9

                // A "1" represents an 16th-note, 2 an eighth note, 4 a quarter note, etc.
                // Rests (spaces) need a length as well.
uint16_t beats[SongLength] = {8,6,2,12,4,4,4,4,4,16,8,6,2,12,4,4,4,4,4,12,3,1,6,2,4,3,1,6,2,4,3,1,4,4,4,4,12,3,1,6,2,4,3,1,6,2,4,4,4,4,2,2,2,2,12};
//uint16_t beats[SongLength] = {32,24,8,48,16,16,16,16,16,16,32,32,24,8,48,16,16,16,16,16,48};
                           //

uint8_t play_song_flag = 0;
uint8_t song_ready;

void play_song_init(void)
{
    uint8_t status;

    for(uint8_t j=0; j<SongLength; j++)
    {
        note[j] = (timer_size_t)(get_period(notes[j]));
        note_duration[j] = (uint32_t)beats[j] * PERIOD16TH * 3;  // length of note/rest in ms
    }
    // Set initial note and note duration
    status = g_playnote_timer.p_api->periodSet(g_playnote_timer.p_ctrl,note[0],TIMER_UNIT_PERIOD_USEC);
    APP_ERR_TRAP(status);
    status = g_notesdur_timer.p_api->periodSet(g_notesdur_timer.p_ctrl, note_duration[0], TIMER_UNIT_PERIOD_MSEC);
    APP_ERR_TRAP(status);
    song_ready = 1;  // flag to indicate song is ready to start
}

uint16_t get_period(char notename)
{
    // This function takes a note character (e.g. 'A', and returns the
    // corresponding period in uS

    static uint8_t i;
    const uint8_t numNotes = 22;  // number of notes stored

    // 1-3 below represent notes below middle C (middle C is represented by 'c' below)
    // v = f#, x = g#, y = c#, Y = C#, n = bflat

           char names[] =     { '3', '2',  '1', 'c', 'd', 'e', 'f',  'g', 'a', 'b', 'C', 'D', 'E', 'F', 'G', 'A', 'B', 'v', 'x', 'y', 'Y', 'n' };
    uint16_t note_periods[] = {5102,4545,4050, 3822,3401,3030,2865,2551,2273,2024,1912,1703,1517,1432,1276,1136,1012,5405,4808,3610,1805,4292};
     //  num of notes      1    2     3    4    5    6    7    8     9    10   11   12   13   14   15   16   17   18   19   20   21    22

    for (i = 0; i < numNotes; i++)
    {
        if (names[i] == notename)
        {
            return (note_periods[i]);
        }
    }
    return (0);

}

void play_song(void)
{
    uint8_t status;
    extern uint8_t justplayed;
    if (note_index == 0 && song_ready ==1 && play_song_flag == 1 && justplayed == 0)
    {
        // Start Song Timers
        status = g_playnote_timer.p_api->start(g_playnote_timer.p_ctrl);
        APP_ERR_TRAP(status);
        status = g_notesdur_timer.p_api->start(g_notesdur_timer.p_ctrl);
        APP_ERR_TRAP(status);
        song_ready = 0;
    }

}

// The g_playnote_timer controls the frequency of the note being played
void g_playnote_timer_callback(timer_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);

    if (notes[note_index] == ' ')          // is this a rest?
    {
        g_ioport.p_api->pinWrite(Speaker_pin, IOPORT_LEVEL_LOW);  // play a pause
    }
    else                          // otherwise, toggle the IO pin to play the note
    {
        g_ioport.p_api->pinWrite(Speaker_pin, level2);  // toggle output
        level2^=1;
    }
    g_playnote_timer.p_api->start(g_playnote_timer.p_ctrl);
}

// The g_notesdur_timer handles the length of play of each note.  This is a oneshot timer
void g_notesdur_timer_callback(timer_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    g_notesdur_timer.p_api->stop(g_notesdur_timer.p_ctrl);
    g_playnote_timer.p_api->stop(g_playnote_timer.p_ctrl);

    g_notepause_timer.p_api->start(g_notepause_timer.p_ctrl);  // adds pause between notes

}

// Timer to add 10ms delay between notes.   Oneshot timer
void g_notepause_timer_callback(timer_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    extern uint8_t justplayed;
    note_index++;
    if (note_index > SongLength - 1)
    {
        note_index = 0;
        play_song_flag = 0;
        song_ready = 1;  // flag to indicate song is ready to start again
        justplayed = 1;
    }
    g_notesdur_timer.p_api->periodSet(g_notesdur_timer.p_ctrl, note_duration[note_index], TIMER_UNIT_PERIOD_MSEC);
    g_notesdur_timer.p_api->start(g_notesdur_timer.p_ctrl);


    g_playnote_timer.p_api->periodSet(g_playnote_timer.p_ctrl,note[note_index],TIMER_UNIT_PERIOD_USEC);
    g_playnote_timer.p_api->start(g_playnote_timer.p_ctrl);

}

